@@ -63,7 +63,7 @@ enum {
  * widget classes.
  */
 
-typedef struct ThreadSpecificData {
+typedef struct {
     BITMAPINFOHEADER *boxesPtr;	/* Information about the bitmap. */
     DWORD *boxesPalette;	/* Pointer to color palette. */
     LPSTR boxesBits;		/* Pointer to bitmap data. */
@@ -86,10 +86,11 @@ static void		InitBoxes(void);
  * The class procedure table for the button widgets.
  */
 
-Tk_ClassProcs tkpButtonProcs = {
+const Tk_ClassProcs tkpButtonProcs = {
     sizeof(Tk_ClassProcs),	/* size */
     TkButtonWorldChanged,	/* worldChangedProc */
     CreateProc,			/* createProc */
+    NULL					/* modalProc */
 };
 
 
@@ -126,13 +127,13 @@ InitBoxes(void)
     HRSRC hrsrc;
     HGLOBAL hblk;
     LPBITMAPINFOHEADER newBitmap;
-    DWORD size;
+    size_t size;
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    hrsrc = FindResource(module, "buttons", RT_BITMAP);
+    hrsrc = FindResourceW(module, L"buttons", (LPWSTR) RT_BITMAP);
     if (hrsrc == NULL) {
-	Tcl_Panic("FindResource() failed for buttons bitmap resource, "
+	Tcl_Panic("FindResourceW() failed for buttons bitmap resource, "
             "resources in tk_base.rc must be linked into Tk dll or static executable");
     } else {
 	hblk = LoadResource(module, hrsrc);
@@ -145,9 +146,10 @@ InitBoxes(void)
 
     if (tsdPtr->boxesPtr != NULL && !(tsdPtr->boxesPtr->biWidth % 4)
 	    && !(tsdPtr->boxesPtr->biHeight % 2)) {
-	size = tsdPtr->boxesPtr->biSize + (1 << tsdPtr->boxesPtr->biBitCount)
-		* sizeof(RGBQUAD) + tsdPtr->boxesPtr->biSizeImage;
-	newBitmap = (LPBITMAPINFOHEADER) ckalloc(size);
+	size = tsdPtr->boxesPtr->biSize
+		+ (sizeof(RGBQUAD) << tsdPtr->boxesPtr->biBitCount)
+		+ tsdPtr->boxesPtr->biSizeImage;
+	newBitmap = ckalloc(size);
 	memcpy(newBitmap, tsdPtr->boxesPtr, size);
 	tsdPtr->boxesPtr = newBitmap;
 	tsdPtr->boxWidth = tsdPtr->boxesPtr->biWidth / 4;
@@ -155,7 +157,7 @@ InitBoxes(void)
 	tsdPtr->boxesPalette = (DWORD*) (((LPSTR) tsdPtr->boxesPtr)
 		+ tsdPtr->boxesPtr->biSize);
 	tsdPtr->boxesBits = ((LPSTR) tsdPtr->boxesPalette)
-	    + ((1 << tsdPtr->boxesPtr->biBitCount) * sizeof(RGBQUAD));
+		+ (sizeof(RGBQUAD) << tsdPtr->boxesPtr->biBitCount);
     } else {
 	tsdPtr->boxesPtr = NULL;
     }
@@ -180,15 +182,12 @@ InitBoxes(void)
  */
 
 void
-TkpButtonSetDefaults(
-    Tk_OptionSpec *specPtr)	/* Points to an array of option specs,
-				 * terminated by one with type
-				 * TK_OPTION_END. */
+TkpButtonSetDefaults()
 {
     int width = GetSystemMetrics(SM_CXEDGE);
-    if (width > 0) {
-	sprintf(tkDefButtonBorderWidth, "%d", width);
-    }
+	if (width > 0) {
+	    sprintf(tkDefButtonBorderWidth, "%d", width);
+	}
 }
 
 /*
@@ -213,7 +212,7 @@ TkpCreateButton(
 {
     WinButton *butPtr;
 
-    butPtr = (WinButton *)ckalloc(sizeof(WinButton));
+    butPtr = ckalloc(sizeof(WinButton));
     butPtr->hwnd = NULL;
     return (TkButton *) butPtr;
 }
@@ -243,24 +242,24 @@ CreateProc(
 {
     Window window;
     HWND parent;
-    char *class;
+    LPCWSTR windowClass;
     WinButton *butPtr = (WinButton *)instanceData;
 
     parent = Tk_GetHWND(parentWin);
     if (butPtr->info.type == TYPE_LABEL) {
-	class = "STATIC";
+	windowClass = L"STATIC";
 	butPtr->style = SS_OWNERDRAW | WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS;
     } else {
-	class = "BUTTON";
+	windowClass = L"BUTTON";
 	butPtr->style = BS_OWNERDRAW | WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS;
     }
-    butPtr->hwnd = CreateWindow(class, NULL, butPtr->style,
+    butPtr->hwnd = CreateWindowW(windowClass, NULL, butPtr->style,
 	    Tk_X(tkwin), Tk_Y(tkwin), Tk_Width(tkwin), Tk_Height(tkwin),
 	    parent, NULL, Tk_GetHINSTANCE(), NULL);
     SetWindowPos(butPtr->hwnd, HWND_TOP, 0, 0, 0, 0,
 		    SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
-    butPtr->oldProc = (WNDPROC)SetWindowLongPtr(butPtr->hwnd, GWLP_WNDPROC,
-	    (INT_PTR) ButtonProc);
+    butPtr->oldProc = (WNDPROC)SetWindowLongPtrW(butPtr->hwnd, GWLP_WNDPROC,
+	    (LONG_PTR) ButtonProc);
 
     window = Tk_AttachHWND(tkwin, butPtr->hwnd);
     return window;
@@ -290,7 +289,7 @@ TkpDestroyButton(
     HWND hwnd = winButPtr->hwnd;
 
     if (hwnd) {
-	SetWindowLongPtr(hwnd, GWLP_WNDPROC, (INT_PTR) winButPtr->oldProc);
+	SetWindowLongPtrW(hwnd, GWLP_WNDPROC, (LONG_PTR) winButPtr->oldProc);
     }
 }
 
@@ -406,7 +405,10 @@ TkpDisplayButton(
      * Compute width of default ring and offset for pushed buttons.
      */
 
-    if (butPtr->type == TYPE_BUTTON) {
+    if (butPtr->type == TYPE_LABEL) {
+	defaultWidth = butPtr->highlightWidth;
+        offset = 0;
+    } else if (butPtr->type == TYPE_BUTTON) {
 	defaultWidth = ((butPtr->defaultState == DEFAULT_ACTIVE)
 		? butPtr->highlightWidth : 0);
 	offset = 1;
@@ -435,7 +437,7 @@ TkpDisplayButton(
      * Display image or bitmap or text for button.
      */
 
-    if (butPtr->image != None) {
+    if (butPtr->image != NULL) {
 	Tk_SizeOfImage(butPtr->image, &width, &height);
 	haveImage = 1;
     } else if (butPtr->bitmap != None) {
@@ -760,17 +762,24 @@ TkpDisplayButton(
 		butPtr->borderWidth, relief);
     }
     if (defaultWidth != 0) {
+        int highlightColor;
+
 	dc = TkWinGetDrawableDC(butPtr->display, pixmap, &state);
+        if (butPtr->type == TYPE_LABEL) {
+            highlightColor = (int) Tk_3DBorderColor(butPtr->highlightBorder)->pixel;
+        } else {
+            highlightColor = (int) butPtr->highlightColorPtr->pixel;
+        }
 	TkWinFillRect(dc, 0, 0, Tk_Width(tkwin), defaultWidth,
-		(int) butPtr->highlightColorPtr->pixel);
+		highlightColor);
 	TkWinFillRect(dc, 0, 0, defaultWidth, Tk_Height(tkwin),
-		(int) butPtr->highlightColorPtr->pixel);
+		highlightColor);
 	TkWinFillRect(dc, 0, Tk_Height(tkwin) - defaultWidth,
 		Tk_Width(tkwin), defaultWidth,
-		(int) butPtr->highlightColorPtr->pixel);
+		highlightColor);
 	TkWinFillRect(dc, Tk_Width(tkwin) - defaultWidth, 0,
 		defaultWidth, Tk_Height(tkwin),
-		(int) butPtr->highlightColorPtr->pixel);
+		highlightColor);
 	TkWinReleaseDrawableDC(pixmap, dc, &state);
     }
 
@@ -1267,29 +1276,37 @@ ButtonProc(
 	return 0;
     }
     case BN_CLICKED: {
-	int code;
-	Tcl_Interp *interp = butPtr->info.interp;
+	/*
+	 * OOPS: chromium fires WM_NULL regularly to ping if plugin is still
+	 * alive. When using an external window (i.e. via the tcl plugin), this
+	 * causes all buttons to fire once a second, so we need to make sure
+	 * that we are not dealing with the chromium life check.
+	*/
+        if (wParam != 0 || lParam != 0) {
+	    int code;
+	    Tcl_Interp *interp = butPtr->info.interp;
 
-	if (butPtr->info.state != STATE_DISABLED) {
-	    Tcl_Preserve((ClientData)interp);
-	    code = TkInvokeButton((TkButton*)butPtr);
-	    if (code != TCL_OK && code != TCL_CONTINUE
-		    && code != TCL_BREAK) {
-		Tcl_AddErrorInfo(interp, "\n    (button invoke)");
-		Tcl_BackgroundError(interp);
+	    if (butPtr->info.state != STATE_DISABLED) {
+		Tcl_Preserve((ClientData)interp);
+		code = TkInvokeButton((TkButton*)butPtr);
+		if (code != TCL_OK && code != TCL_CONTINUE
+			&& code != TCL_BREAK) {
+		    Tcl_AddErrorInfo(interp, "\n    (button invoke)");
+		    Tcl_BackgroundException(interp, code);
+		}
+		Tcl_Release((ClientData)interp);
 	    }
-	    Tcl_Release((ClientData)interp);
+	    Tcl_ServiceAll();
+	    return 0;
 	}
-	Tcl_ServiceAll();
-	return 0;
     }
-
+    /* FALLTHRU */
     default:
 	if (Tk_TranslateWinEvent(hwnd, message, wParam, lParam, &result)) {
 	    return result;
 	}
     }
-    return DefWindowProc(hwnd, message, wParam, lParam);
+    return DefWindowProcW(hwnd, message, wParam, lParam);
 }
 
 /*
